#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <libps/pslib.h>
#ifdef MEMORY_DEBUGGING
#include <libps/pslib-mp.h>
#endif

void errorhandler(PSDoc *p, int error, const char *str, void *data) {
	fprintf(stderr, "PSLib: %s\n", str);
}

size_t writeproc(PSDoc *p, void *data, size_t size) {
	return fwrite(data, 1, size, stdout);
}

#define LEFT_BORDER 50

int main() {
	PSDoc *ps;
	int psfont;
	int psimage;

	PS_boot();
#ifdef MEMORY_DEBUGGING
	PS_mp_init();
#endif

#ifdef MEMORY_DEBUGGING
	ps = PS_new2(errorhandler, PS_mp_malloc, PS_mp_realloc, PS_mp_free, NULL);
#else
	ps = PS_new();
#endif

	if (0 > PS_open_file(ps, "image.ps")) {
		printf("Cannot open PostScript file\n");
		exit(1);
	}

	PS_set_parameter(ps, "warning", "true");
	PS_set_parameter(ps, "imageencoding", "hex");

	PS_set_info(ps, "Creator", __FILE__);
	PS_set_info(ps, "Author", "Uwe Steinmann");
	PS_set_info(ps, "Title", "Image Example");
	PS_set_info(ps, "Keywords", "Image Picture PNG");

	PS_begin_page(ps, 596, 842);
	psfont = PS_findfont(ps, "Helvetica", "", 0);
	PS_setfont(ps, psfont, 8.0);

	psimage = PS_open_image_file(ps, "png", "indexed.png", NULL, 0);
	PS_place_image(ps, psimage, 50, 730, 1.0);
	PS_show_xy(ps, "indexed image without alpha channel", 50, 710);

	psimage = PS_open_image_file(ps, "png", "rgb.png", NULL, 0);
	PS_place_image(ps, psimage, 300, 730, 1.0);
	PS_show_xy(ps, "rgb image without alpha channel", 300, 710);

	psimage = PS_open_image_file(ps, "png", "indexed-alpha.png", NULL, 0);
	PS_place_image(ps, psimage, 50, 630, 1.0);
	PS_show_xy(ps, "indexed image with alpha channel", 50, 610);

	psimage = PS_open_image_file(ps, "png", "rgb-alpha.png", NULL, 0);
	PS_place_image(ps, psimage, 300, 630, 1.0);
	PS_show_xy(ps, "rgb image with alpha channel", 300, 610);

	PS_end_page(ps);

	PS_close(ps);
	PS_delete(ps);

#ifdef MEMORY_DEBUGGING 
	PS_mp_list_unfreed();
#endif  

	PS_shutdown();
	exit(0);
}
